function [Af, crit] = granger_songsiri(Y, p, mask)
% VAR model identification with Granger constraints
% Stability imposed like in Rakasri & Songsiri 2016
%
% Input:
%   Y          - data matrix
%   p          - AR order
%   mask       - zero mask: KxK matrix with 1 in the positions where the
%                coefficients are zero, and zeros elsewhere
%
% Return:
%   Af         - the resulting VAR model, as a cell. (The first coefficient
%                is implicitly I and is not saved.)
%   crit       - the optimal value of the LS criterion

% BD 14.02.2018

tol_stab = 0.999;

[K,N] = size(Y);

% build matrix for LS objective
H = zeros(K*p, N-p);
for i = 1 : N-p
  H(:,i) = vec(Y(:,p+i-1:-1:i));
end

ncoef = p*K*K;                  % number of coefficients of the AR model

imask = find(vec(mask));        % zero positions in vectorized coefficient
cvx_begin
  cvx_quiet('true');
  variable A(ncoef);
  minimize norm(Y(:,p+1:N) - reshape(A, K, K*p) * H, 'fro');
  subject to
    norm(reshape(A, K, K*p), inf) <= tol_stab;  % inf-norm stability
    % zero constraints
    i = 0;
    for k = 1:p
      A(i+imask) == 0;
      i = i + K*K;
    end
cvx_end

crit = cvx_optval;

% build final result
%Af{1} = eye(K);
i = 0;
for k = 1 : p
  Af{k} = reshape(A(i+1:i+K*K), K, K);
  i = i + K*K;
end
