function [Af, crit] = granger_stab(Y, p, mask, A, tol_pos)

% Input:
%   Y          - data matrix
%   p          - AR order
%   mask       - zero mask: KxK matrix with 1 in the positions where the
%                coefficients are zero, and zeros elsewhere
%   A          - stable polynomial around which the stability domain is built;
%                first coefficient is forced to I;
%   tol_pos    - tolerance to positivity constraint; should be larger for
%                large systems; may be nearly zero (e.g. 1e-5) for small systems
%
% Return:
%   Af         - the resulting VAR model, as a cell. (The first coefficient
%                is implicitly I and is not saved.)
%   crit       - the optimal value of the LS criterion

% BD 13.02.2018

[K,N] = size(Y);

% build matrix for LS objective
H = zeros(K*p, N-p);
for i = 1 : N-p
  H(:,i) = vec(Y(:,p+i-1:-1:i));
end

K2 = K*(K+1)/2;                 % number of distinct coefficients in the symmetric term
ncoef = p*K*K;                  % number of coefficients of the AR model
ncoef_sim = K2 + ncoef;         % number of coefficients for the symmetric polynomial in positive realness condition

A{1} = eye(K);                  % force first coefficient to unit matrix
Av = zeros(ncoef,1);            % vectorized matrix polynomial
i = 0;
for k = 2:p+1
  Av(i+1:i+K*K) = vec(A{k});
  i = i + K*K;
end

% Pos3Poly problem
pars.fid = 0;                   % don't show any message
Xp = [p K];                     % descriptor for the symmetric matrix polynomial
Xptype.trigonometric = 1;

%C = zeros(ncoef_sim, ncoef);    % describes the linear relation between X and D
C = sparse(ncoef_sim, ncoef);    % describes the linear relation between X and D
lin = K2 : K*K : ncoef_sim;     % where the blocks of C start...
col = 0 : K*K : ncoef;
%Q = TvecMat(K,K);               % Q*vec(Dj) = vec(Dj')
Q = sparse(TvecMat(K,K));       % Q*vec(Dj) = vec(Dj')
for k = 1:p                     % non-central coefficients
  C(lin(k)+1:lin(k+1), col(k)+1:col(k+1)) = eye(K*K);
  for j = 1 : p-k
%    C(lin(k)+1:lin(k+1), col(j+k-1)+1:col(j+k)) = C(lin(k)+1:lin(k+1), col(j+k-1)+1:col(j+k)) + kron(eye(K), A{j}');
%    C(lin(k)+1:lin(k+1), col(j)+1:col(j+1)) = C(lin(k)+1:lin(k+1), col(j)+1:col(j+1)) + kron(A{j+k}', eye(K))*Q;
    C(lin(k)+1:lin(k+1), col(j+k-1)+1:col(j+k)) = C(lin(k)+1:lin(k+1), col(j+k-1)+1:col(j+k)) + sparse(kron(eye(K), A{j}'));
    C(lin(k)+1:lin(k+1), col(j)+1:col(j+1)) = C(lin(k)+1:lin(k+1), col(j)+1:col(j+1)) + sparse(kron(A{j+k}', eye(K))*Q);
  end
end
% the central coefficient
%lins = vecs(mat(1:K*K));
lins = vltri(mat(1:K*K));
for j = 1 : p
%  Ct = kron(eye(K),A{j}') + kron(A{j}',eye(K)) * Q;
%  C(1:K2, col(j)+1:col(j+1)) = Ct(lins,:);
  Ct = sparse(kron(eye(K),A{j}') + kron(A{j}',eye(K)) * Q);
  C(1:K2, col(j)+1:col(j+1)) = sparse(Ct(lins,:));
end

% the free term
for k = 0 : p
  Atr{k+1} = A{p+1-k}';
end
Asim = mult(Atr,A);
v = zeros(ncoef_sim,1);
%v(1:K2) = vecs(Asim{p+1});
v(1:K2) = vltri(Asim{p+1});
for k = 1 : p
  v(lin(k)+1:lin(k+1)) = vec(Asim{p+k+1});
end
v = 2*v;  % !!!

imask = find(vec(mask));        % zero positions in vectorized coefficient
cvx_begin
  cvx_quiet('true');
%  cvx_solver sedumi;
  variable X(ncoef_sim);        % symmetric polynomial in positive realness condition
  variable D(ncoef);
  minimize norm(Y(:,p+1:N) - reshape(Av+D, K, K*p) * H, 'fro');
  subject to
    % positivity constraint
    X - tol_pos*unitpol(p,K) == sos_pol(Xp, Xptype, pars);   % X is positive matrix polynomial
    X == C * D + v;
    
    % zero constraints
    i = 0;
    for k = 1:p
      Av(i+imask) + D(i+imask) == 0;
      i = i + K*K;
    end
cvx_end

crit = cvx_optval;

% build final result
%Af{1} = eye(K);
i = 0;
for k = 1 : p
  Af{k} = reshape(Av(i+1:i+K*K) + D(i+1:i+K*K), K, K);
  i = i + K*K;
end
