function [val_crit, val_spec_rad, val_coeff, vec_diff] = opt_algo(data, phat, masktrue1, est_mask, nit, tol_pos, NoITC)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Computes various results 
%Input:
% data          = measurements (K X N)
% phat          = est. order
% maske_xor     = xor mask
% masktrue1     = 1 - masktrue
% est_mask      = candidates for mask
% nit           = no. of iterations
% tol_pos       = improves numerical conditioning
% NoITC         = total no. of ITC for the selection of the mask
%Output:
% val_crit      = values for the ITC                            [no. of ITC X no. of candidates X no. of opt. methods]
% val_spec_rad  = values spec. rad.                             [no. of candidates X no. of opt. methods]
% val_coeff     = estimated coeff.(cell)                        [no. of candidates X no. of opt. methods]  
% vec_diff      = dist. from each candidate to the masktrue1    [1 X no. of candidates]
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

val_crit        = Inf*ones(NoITC,size(est_mask,2),3);
val_spec_rad    = Inf*ones(size(est_mask,2),3);
val_coeff       = cell(size(est_mask,2),3);
vec_diff        = Inf*ones(1,size(est_mask,2));

K               = size(data,1);
noz_old         = Inf;
for ind_mask=1:size(est_mask,2)
    maske = est_mask{ind_mask};
    noz = K^2-sum(sum(maske));
    vec_diff(ind_mask) = sum(sum(xor(maske,masktrue1)));
    if (noz>0) && (noz<K^2)
        if noz~=noz_old
            noz_old = noz;
            maske_xor = xor(maske, ones(size(maske)));
            
            % simple LS solution
            met = 1;
            [Afls, ~] = granger_simple(data, phat, maske_xor);
            val_coeff{ind_mask, met} = Afls;
            [val_spec_rad, val_crit] = comp_res(val_spec_rad, val_crit, data', Afls, noz, met, ind_mask);
            
            % Songsiri's solution
            met = 2;
            [Afsong, ~] = granger_songsiri(data, phat, maske_xor);
            val_coeff{ind_mask, met} = Afsong;
            [val_spec_rad, val_crit] = comp_res(val_spec_rad, val_crit, data', Afsong, noz, met, ind_mask);
            
            % Stability condition
            met     = 3;
            [Af, ~] = run_granger_stab(data, phat, maske_xor, nit, tol_pos, Afsong);
            val_coeff{ind_mask, met} = Af;
            [val_spec_rad, val_crit] = comp_res(val_spec_rad, val_crit, data', Af, noz, met, ind_mask);
        else
            val_spec_rad(ind_mask,:) = val_spec_rad(ind_mask-1,:);
            val_crit(:,ind_mask,:) = val_crit(:,ind_mask-1,:);
            val_coeff(ind_mask,:) = val_coeff(ind_mask-1,:); %modified Aug. 21, 2018
        end %if comp. prev. noz
    end %if valid noz 

end %for





end %function

